package core

import (
	"grid-prover/database"
	"grid-prover/logs"
	"math/big"
	"strings"

	"github.com/ethereum/go-ethereum/accounts/abi"
	"github.com/ethereum/go-ethereum/common"
)

var (
	// blockNumber = big.NewInt(0)
	logger = logs.Logger("dumper")
)

type Dumper struct {
	endpoint        string
	contractABI     []abi.ABI
	contractAddress []common.Address
	// store           MapStore

	blockNumber *big.Int

	eventNameMap map[common.Hash]string
	indexedMap   map[common.Hash]abi.Arguments
}

func NewGRIDDumper(chain string, contractAddress common.Address) (dumper *Dumper, err error) {
	dumper = &Dumper{
		// store:        store,
		eventNameMap: make(map[common.Hash]string),
		indexedMap:   make(map[common.Hash]abi.Arguments),
	}

	dumper.contractAddress = []common.Address{contractAddress}

	fpContractABI, err := abi.JSON(strings.NewReader(GRIDABI))
	if err != nil {
		return dumper, err
	}

	dumper.contractABI = []abi.ABI{fpContractABI}

	for _, ABI := range dumper.contractABI {
		for name, event := range ABI.Events {
			dumper.eventNameMap[event.ID] = name
			var indexed abi.Arguments
			for _, arg := range ABI.Events[name].Inputs {
				if arg.Indexed {
					indexed = append(indexed, arg)
				}
			}
			dumper.indexedMap[event.ID] = indexed
		}
	}

	blockNumber, err := database.GetBlockNumber()
	if err != nil {
		blockNumber = 0
	}
	dumper.blockNumber = big.NewInt(blockNumber)

	return dumper, nil
}
