package database

import "time"

type Profit struct {
	Address string `gorm:"primarykey"` // CPU/GPU供应商ID
	// ID       int64
	Balance  int64     // 余额
	Profit   int64     // 分润值
	Penalty  int64     // 惩罚值
	LastTime time.Time // 上次更新时间
	EndTime  time.Time // 可以取出全部分润值时间
}

func InitProfit() error {
	return GlobalDataBase.AutoMigrate(&Profit{})
}

func (p *Profit) CreateProfit() error {
	return GlobalDataBase.Create(p).Error
}

func (p *Profit) UpdateProfit() error {
	return GlobalDataBase.Model(&Profit{}).Where("address = ?", p.Address).Save(p).Error
}

func GetProfitByAddress(address string) (Profit, error) {
	var profit Profit
	err := GlobalDataBase.Model(&Profit{}).Where("address = ?", address).First(&profit).Error
	if err != nil {
		return Profit{}, err
	}

	return profit, nil
}

var blockNumberKey = "block_number_key"

type BlockNumber struct {
	BlockNumberKey string `gorm:"primarykey;column:key"`
	BlockNumber    int64
}

func SetBlockNumber(blockNumber int64) error {
	var daBlockNumber = BlockNumber{
		BlockNumberKey: blockNumberKey,
		BlockNumber:    blockNumber,
	}
	return GlobalDataBase.Save(&daBlockNumber).Error
}

func GetBlockNumber() (int64, error) {
	var blockNumber BlockNumber
	err := GlobalDataBase.Model(&BlockNumber{}).First(&blockNumber).Error

	return blockNumber.BlockNumber, err
}
