package database

type Provider struct {
	Address string `gorm:"primarykey"`
	Name    string
	IP      string
	Port    string
}

type Node struct {
	Address  string `gorm:"primaryKey"`
	Id       int    `gorm:"primaryKey"`
	CPUPrice int64
	CPUModel string

	GPUPrice int64
	GPUModel string

	MemPrice    int64
	MemCapacity int64

	DiskPrice    int64
	DiskCapacity int64
}

func InitProvider() error {
	return GlobalDataBase.AutoMigrate(&Provider{})
}

func (p *Provider) CreateProvider() error {
	return GlobalDataBase.Create(p).Error
}

func (p *Provider) GetProviderByAddress(address string) (Provider, error) {
	var provider Provider
	err := GlobalDataBase.Model(&Provider{}).Where("address = ?", address).First(&provider).Error
	if err != nil {
		return Provider{}, err
	}

	return provider, nil
}

func InitNode() error {
	return GlobalDataBase.AutoMigrate(&Node{})
}

func (n *Node) CreateNode() error {
	return GlobalDataBase.Create(n).Error
}

func (n *Node) GetNodeByAddressAndId(address string, id int64) (Node, error) {
	var node Node
	err := GlobalDataBase.Model(&Node{}).Where("address = ? AND id = ?", address, id).First(&node).Error
	if err != nil {
		return node, err
	}

	return node, nil
}
